@@ -45,7 +45,6 @@ Box_Data_List::Box_Data_List()
 
   pack_start(m_AddDel);
   m_AddDel.set_auto_add(false); //We want to add the row ourselves when the user clicks the Add button, because the default behaviour there is not suitable.
-  m_AddDel.set_allow_column_chooser();
   m_AddDel.set_rules_hint(); //Use alternating row colors when the theme does that.
   
   //Connect signals:
@@ -55,6 +54,8 @@ Box_Data_List::Box_Data_List()
   m_AddDel.signal_user_added().connect(sigc::mem_fun(*this, &Box_Data_List::on_adddel_user_added));
   m_AddDel.signal_user_changed().connect(sigc::mem_fun(*this, &Box_Data_List::on_adddel_user_changed));
   m_AddDel.signal_user_reordered_columns().connect(sigc::mem_fun(*this, &Box_Data_List::on_adddel_user_reordered_columns));
+  
+  m_AddDel.signal_user_requested_layout().connect(sigc::mem_fun(*this, &Box_Data_List::on_adddel_user_requested_layout));
 
 
   //Groups are not very helpful for a list view:
@@ -90,14 +91,19 @@ void Box_Data_List::fill_from_database()
       if(!m_strWhereClause.empty())
         strWhereClausePiece = " WHERE " + m_strWhereClause;
 
-      type_vecFields listFieldsToShow = get_fields_to_show();
-      m_Fields = listFieldsToShow; //Store the fields for later:
-      if(!m_Fields.empty())
+      m_Fields = get_fields_to_show();
+
+      //Add extra possibly-non-visible columns that we need:
+      type_vecFields fieldsToGet = m_Fields;
+      fieldsToGet.push_back(m_AddDel.get_key_field());
+      const int index_primary_key = fieldsToGet.size() - 1;
+
+      if(!fieldsToGet.empty())
       {
         Glib::ustring sql_part_fields;
-        for(type_vecFields::const_iterator iter =  listFieldsToShow.begin(); iter != listFieldsToShow.end(); ++iter)
+        for(type_vecFields::const_iterator iter =  fieldsToGet.begin(); iter != fieldsToGet.end(); ++iter)
         {
-          if(iter != listFieldsToShow.begin())
+          if(iter != fieldsToGet.begin())
             sql_part_fields += ",";
 
           sql_part_fields += iter->get_name();
@@ -122,53 +128,48 @@ void Box_Data_List::fill_from_database()
           if(rows_count > 100)
             rows_count = 100; //Don't get more than 100. TODO: Get other rows dynamically.
 
-          guint primary_key_field_index = 0;
-          bool primary_key_found = get_field_primary_key(primary_key_field_index);
-          if(primary_key_found)
+          for(guint result_row = 0; result_row < (rows_count); result_row++)
           {
-            for(guint result_row = 0; result_row < rows_count; result_row++)
+            Gnome::Gda::Value key = result->get_value_at(index_primary_key, result_row);
+            //It is usually an integer.
+            if(GlomConversions::value_is_empty(key))
+              g_warning("Box_Data_List::fill_from_database(): primary key value is empty");
+            else
             {
+              Gnome::Gda::Value value_primary_key = result->get_value_at(index_primary_key, result_row);
 
-              Gnome::Gda::Value key = result->get_value_at(primary_key_field_index, result_row);
-              //It is usually an integer.
-              if(GlomConversions::value_is_empty(key))
-                g_warning("Box_Data_List::fill_from_database(): primary key value is empty");
-              else
+              Gtk::TreeModel::iterator tree_iter = m_AddDel.add_item(value_primary_key);
+
+              type_vecFields::const_iterator iterFields = fieldsToGet.begin();
+
+              //each field:
+              //We use cols_count -1 because we have an extra field for the primary_key.
+              guint cols_count = result->get_n_columns() - 1;
+              for(guint uiCol = 0; uiCol < cols_count; uiCol++)
               {
-                Gtk::TreeModel::iterator tree_iter = m_AddDel.add_item(key);
+                Gnome::Gda::Value value = result->get_value_at(uiCol, result_row);
+                guint index = 0;
 
-                type_vecFields::const_iterator iterFields = listFieldsToShow.begin();
+                //g_warning("list fill: field_name=%s", iterFields->get_name().c_str());
+                //g_warning("  value_as_string=%s", value.to_string().c_str());
 
-                //each field:
-                guint cols_count = result->get_n_columns();
-                for(guint uiCol = 0; uiCol < cols_count; uiCol++)
+                //TODO_Performance: This searches fieldsToGet again each time:
+                const Glib::ustring field_name = iterFields->get_name();
+                bool test = get_field_column_index(field_name, index);
+                ++iterFields;
+
+                if(test)
                 {
-                  Gnome::Gda::Value value = result->get_value_at(uiCol, result_row);
-                  guint index = 0;
-
-                  //g_warning("list fill: field_name=%s", iterFields->get_name().c_str());
-                  //g_warning("  value_as_string=%s", value.to_string().c_str());
-
-                  //TODO_Performance: This searches m_Fields again each time:
-                  const Glib::ustring field_name = iterFields->get_name();
-                  bool test = get_field_column_index(field_name, index);
-                  ++iterFields;
-
-                  if(test)
-                  {
-                    m_AddDel.set_value(tree_iter, index, value);
-                    //g_warning("addedel size=%d", m_AddDel.get_count());
-                  }
-		  else
-		    g_warning("  get_field_column_index failed: field name=%s", field_name.c_str());
+                  m_AddDel.set_value(tree_iter, index, value);
+                  //g_warning("addedel size=%d", m_AddDel.get_count());
                 }
+                else
+                  g_warning("  get_field_column_index failed: field name=%s", field_name.c_str());
               }
             }
           }
-          else
-            g_warning("Box_Data_List::fill_from_database(): primary key not found in table %s", m_strTableName.c_str());
         }
-      } //If !m_Fields.empty()
+      } //If !fieldsToGet.empty()
     }
    
     //Select first record:
@@ -192,7 +193,7 @@ void Box_Data_List::on_adddel_user_requested_add()
   {
     //Start editing in the primary key or the first cell if the primary key is auto-incremented (because there is no point in editing an auto-generated value)..
     guint index_primary_key = 0;
-    bool bPresent = get_field_primary_key(index_primary_key); //If there is no primary key then the default of 0 is OK.
+    bool bPresent = get_field_primary_key_index(index_primary_key); //If there is no primary key then the default of 0 is OK.
     guint index_field_to_edit = 0;
     if(bPresent)
     {
@@ -230,38 +231,52 @@ void Box_Data_List::on_adddel_user_requested_add()
 
 void Box_Data_List::on_adddel_user_requested_edit(const Gtk::TreeModel::iterator& row)
 {
-  Gnome::Gda::Value primary_key_value = m_AddDel.get_value_key_as_value(row); //The primary key is in the key.
+  Gnome::Gda::Value primary_key_value = m_AddDel.get_value_key(row); //The primary key is in the key.
 
   signal_user_requested_details().emit(primary_key_value);
 }
 
+void Box_Data_List::on_adddel_user_requested_layout()
+{
+  show_layout_dialog();
+}
+
 void Box_Data_List::on_adddel_user_requested_delete(const Gtk::TreeModel::iterator& rowStart, const Gtk::TreeModel::iterator&  /* rowEnd TODO */)
 {
-  record_delete( get_primary_key_value(rowStart) );
+  if(rowStart)
+  {
+    //Ask the user for confirmation:
+    Gtk::MessageDialog dialog(gettext("Are you sure that you would like to delete this record? The data in this record will then be permanently lost."), false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE);
+    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+    dialog.add_button(Gtk::Stock::DELETE, Gtk::RESPONSE_OK);
+    
+    int response = dialog.run();
+    if(response == Gtk::RESPONSE_OK)
+    {
+      record_delete( get_primary_key_value(rowStart) );
 
-  //Remove the row:
-  m_AddDel.remove_item(rowStart);
+      //Remove the row:
+      m_AddDel.remove_item(rowStart);
+    }
+  }
 }
 
 void Box_Data_List::on_adddel_user_added(const Gtk::TreeModel::iterator& row)
 {
   Gnome::Gda::Value primary_key_value;
 
-  const Glib::ustring& strPrimaryKeyName = get_primarykey_name();
-  Field field;
-  bool found = get_field(strPrimaryKeyName, field);
-  if(!found)
-    g_warning("Box_Data_List::on_adddel_user_added(): primary key %s not found.", strPrimaryKeyName.c_str());
+  Field field_primary_key = m_AddDel.get_key_field();
 
   guint generated_id = 0;
-  if(field.get_field_info().get_auto_increment())
+  if(field_primary_key.get_field_info().get_auto_increment())
   {
     //Auto-increment is awkward (we can't get the last-generated ID) with postgres, so we auto-generate it ourselves;
+    const Glib::ustring& strPrimaryKeyName = field_primary_key.get_name();
     generated_id = generate_next_auto_increment(m_strTableName, strPrimaryKeyName);  //TODO: return a Gnome::Gda::Value of an appropriate type.
     Glib::ustring strPrimaryKeyValue = util_string_from_decimal(generated_id);
 
     bool parsed = false;
-    primary_key_value = GlomConversions::parse_value(field.get_glom_type(), strPrimaryKeyValue, parsed);
+    primary_key_value = GlomConversions::parse_value(field_primary_key.get_glom_type(), strPrimaryKeyValue, parsed);
   }
   else
   {
@@ -270,25 +285,28 @@ void Box_Data_List::on_adddel_user_added(const Gtk::TreeModel::iterator& row)
 
   sharedptr<SharedConnection> sharedconnection = connect_to_server(); //Keep it alive while we need the data_model.
   if(sharedconnection)
-  {      
+  {
     Glib::RefPtr<Gnome::Gda::DataModel> data_model = record_new(true /* use entered field data*/, primary_key_value);
     if(data_model)
     {
+      //Save the primary key value for later use:
+      m_AddDel.set_value_key(row, primary_key_value);
+
+      //Show the primary key in the row, if the primary key is visible:
       guint primary_key_model_col_index = 0;
-      bool test = get_field_column_index(field.get_name(), primary_key_model_col_index);
+      bool test = get_field_column_index(field_primary_key.get_name(), primary_key_model_col_index);
       if(test)
       {
-        const Gnome::Gda::FieldAttributes fieldInfo = field.get_field_info();
+        const Gnome::Gda::FieldAttributes fieldInfo = field_primary_key.get_field_info();
         //If it's an auto-increment, then get the value and show it:
         if(fieldInfo.get_auto_increment())
         {
-          m_AddDel.set_value_key(row, primary_key_value); //The AddDel key is always a string.
           m_AddDel.set_value(row, primary_key_model_col_index, primary_key_value);
         }
-
-        //Allow derived class to respond to record addition.
-        on_record_added(primary_key_value);
       }
+
+      //Allow derived class to respond to record addition.
+      on_record_added(primary_key_value);
     }
     else
       handle_error();
@@ -336,36 +354,27 @@ void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row,
     //Just update the record:
     try
     {
-      guint primary_key_field_index = 0;
-      bool bPresent = get_field_primary_key(primary_key_field_index);
-      if(bPresent)
-      {
-        Field field_primary_key = m_Fields[primary_key_field_index];
+      Field field_primary_key = m_AddDel.get_key_field();
 
-        const Glib::ustring field_name = m_AddDel.get_column_field(col);
-        Field field;
-        bool test = get_fields_for_table_one_field(m_strTableName, field_name, field);
-        if(test)
-        {        
-          const Gnome::Gda::Value field_value = m_AddDel.get_value_as_value(row, col);
-             
-          Glib::ustring strQuery = "UPDATE " + m_strTableName;
-          strQuery += " SET " +  field.get_name() + " = " + field.sql(field_value);
+      const Glib::ustring field_name = m_AddDel.get_column_field(col);
+      Field field;
+      bool test = get_fields_for_table_one_field(m_strTableName, field_name, field);
+      if(test)
+      {        
+        const Gnome::Gda::Value field_value = m_AddDel.get_value(row, col);
+            
+        Glib::ustring strQuery = "UPDATE " + m_strTableName;
+        strQuery += " SET " +  field.get_name() + " = " + field.sql(field_value);
 
-          strQuery += " WHERE " + field_primary_key.get_name() + " = " + field_primary_key.sql(primary_key_value);
-          Query_execute(strQuery);  //TODO: Handle errors
+        strQuery += " WHERE " + field_primary_key.get_name() + " = " + field_primary_key.sql(primary_key_value);
+        Query_execute(strQuery);  //TODO: Handle errors
 
-          //Get-and-set values for lookup fields, if this field triggers those relationships:
-          do_lookups(row, field, field_value, field_primary_key, primary_key_value);
-        }
-        else
-        {
-          g_warning("Box_Data_List::on_adddel_user_changed(): get_fields_for_table_one_field() failed: m_strTableName=%s, field_name=%s", m_strTableName.c_str(), field_name.c_str());
-        }
+        //Get-and-set values for lookup fields, if this field triggers those relationships:
+        do_lookups(row, field, field_value, field_primary_key, primary_key_value);
       }
       else
       {
-        g_warning("Box_Data_List::on_adddel_user_changed(): primary key not found");
+        g_warning("Box_Data_List::on_adddel_user_changed(): get_fields_for_table_one_field() failed: m_strTableName=%s, field_name=%s", m_strTableName.c_str(), field_name.c_str());
       }
     }
     catch(const std::exception& ex)
@@ -378,10 +387,7 @@ void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row,
     //This record probably doesn't exist yet.
     //Add new record, which will generate the primary key:
 
-    Field field_primary_key;
-
-    bool found = get_field(get_primarykey_name(), field_primary_key);
-    if(found && field_primary_key.get_field_info().get_auto_increment())
+    if(m_AddDel.get_key_field().get_field_info().get_auto_increment())
     {
       on_adddel_user_added(row);
 
@@ -435,7 +441,7 @@ void Box_Data_List::on_details_nav_first()
 {
   m_AddDel.select_item(m_AddDel.get_model()->children().begin());
 
-  signal_user_requested_details().emit(m_AddDel.get_value_key_selected_as_value());
+  signal_user_requested_details().emit(m_AddDel.get_value_key_selected());
 }
 
 void Box_Data_List::on_details_nav_previous()
@@ -449,7 +455,7 @@ void Box_Data_List::on_details_nav_previous()
       iter--;
 
       m_AddDel.select_item(iter);
-      signal_user_requested_details().emit(m_AddDel.get_value_key_selected_as_value());
+      signal_user_requested_details().emit(m_AddDel.get_value_key_selected());
     }
   }
 }
@@ -465,7 +471,7 @@ void Box_Data_List::on_details_nav_next()
       iter++;    
       m_AddDel.select_item(iter);
 
-      signal_user_requested_details().emit(m_AddDel.get_value_key_selected_as_value());
+      signal_user_requested_details().emit(m_AddDel.get_value_key_selected());
     }
   }
 }
@@ -476,7 +482,7 @@ void Box_Data_List::on_details_nav_last()
   if(iter)
   {
     m_AddDel.select_item(iter);
-    signal_user_requested_details().emit(m_AddDel.get_value_key_selected_as_value());
+    signal_user_requested_details().emit(m_AddDel.get_value_key_selected());
   }
 }
 
@@ -515,12 +521,12 @@ void Box_Data_List::on_Details_record_deleted(Gnome::Gda::Value primary_key_valu
 
 Gnome::Gda::Value Box_Data_List::get_primary_key_value(const Gtk::TreeModel::iterator& row)
 {
-  return m_AddDel.get_value_key_as_value(row);
+  return m_AddDel.get_value_key(row);
 }
 
 Gnome::Gda::Value Box_Data_List::get_primary_key_value_selected()
 {
-  return Gnome::Gda::Value(m_AddDel.get_value_key_selected_as_value());
+  return Gnome::Gda::Value(m_AddDel.get_value_key_selected());
 }
 
 Gnome::Gda::Value Box_Data_List::get_entered_field_data(const Field& field) const
@@ -530,7 +536,7 @@ Gnome::Gda::Value Box_Data_List::get_entered_field_data(const Field& field) cons
   guint index = 0;
   bool test = get_field_column_index(field.get_name(), index);
   if(test)
-    return m_AddDel.get_value_selected_as_value(index);
+    return m_AddDel.get_value_selected(index);
   else
     return Gnome::Gda::Value(); //null.
 }
@@ -552,22 +558,26 @@ void Box_Data_List::fill_column_titles()
 {
   const Document_Glom* pDoc = dynamic_cast<const Document_Glom*>(get_document());
   if(pDoc)
-  {    
+  {
     //Field Names:
     m_AddDel.remove_all_columns();
     //m_AddDel.set_columns_count(m_Fields.size());
 
+    Field field_primary_key;
+    bool test = get_field_primary_key_for_table(m_strTableName, field_primary_key);
+    if(test)
+        m_AddDel.set_key_field(field_primary_key);
+
+
     type_vecFields listFieldsToShow = get_fields_to_show();
 
     //Add a column for each table field:
     for(type_vecFields::const_iterator iter =  listFieldsToShow.begin(); iter != listFieldsToShow.end(); ++iter)
     {
       m_AddDel.add_column(*iter);
-
-      if(iter->get_field_info().get_primary_key())
-        m_AddDel.set_key_type(*iter);
     }
   }
+
 }
 
 void Box_Data_List::on_record_added(const Gnome::Gda::Value& /* strPrimaryKey */)
@@ -598,6 +608,16 @@ bool Box_Data_List::get_field_column_index(const Glib::ustring& field_name, guin
     ++i;
   }
 
-  g_warning("Box_Data_List::get_field_column_index(): field not found.");
   return false; //failure.
 }
+
+bool Box_Data_List::get_field_primary_key(Field& field) const
+{
+  field = m_AddDel.get_key_field();
+  return false;
+}
+
+bool Box_Data_List::get_field_primary_key_index(guint& field_column) const
+{
+  return Box_Data::get_field_primary_key_index(m_Fields, field_column);
+}
