@@ -77,8 +77,6 @@ AddDel::AddDel()
 
   m_bAllowUserActions = true;
 
-  m_strSelectText = gettext("Edit");
-
   //Start with a useful default TreeModel:
   //set_columns_count(1);
   //construct_specified_columns();
@@ -183,7 +181,7 @@ void AddDel::setup_menu()
   m_MenuPopup.items().clear();
 
   //Add new menu items:
-  Gtk::Menu_Helpers::MenuElem menuItem_Edit(m_strSelectText);
+  Gtk::Menu_Helpers::MenuElem menuItem_Edit(gettext("Edit"));
   m_MenuPopup.items().push_back(menuItem_Edit);
   m_MenuPopup.items().back().signal_activate().connect(sigc::mem_fun(*this, &AddDel::on_MenuPopup_activate_Edit));
 
@@ -892,16 +890,6 @@ bool AddDel::get_allow_user_actions() const
   return m_bAllowUserActions;
 }
 
-void AddDel::set_select_text(const Glib::ustring& strVal)
-{
-  m_strSelectText = strVal;
-}
-
-Glib::ustring AddDel::get_select_text() const
-{
-  return m_strSelectText;
-}
-
 void AddDel::set_show_column_titles(bool bVal)
 {
   m_TreeView.set_headers_visible(bVal);
@@ -1249,27 +1237,30 @@ void AddDel::on_treeview_button_press_event(GdkEventButton* event)
     Gtk::TreeView::Column* pColumn = 0;
     int cell_x = 0;
     int cell_y = 0;  
-    m_TreeView.get_path_at_pos((int)event->x, (int)event->y, path, pColumn, cell_x, cell_y);
-    
-    //Get the row:
-    Gtk::TreeModel::iterator iterRow = m_refListStore->get_iter(path);
-    if(iterRow)
+    bool row_exists = m_TreeView.get_path_at_pos((int)event->x, (int)event->y, path, pColumn, cell_x, cell_y);
+
+    if(row_exists)
     {
-      //Get the column:
-      int tree_col = 0;
-      int col_index = get_count_hidden_system_columns();
-
-      typedef std::vector<Gtk::TreeView::Column*> type_vecTreeViewColumns;
-      type_vecTreeViewColumns vecColumns = m_TreeView.get_columns();
-      for(type_vecTreeViewColumns::const_iterator iter = vecColumns.begin(); iter != vecColumns.end(); iter++)
+      //Get the row:
+      Gtk::TreeModel::iterator iterRow = m_refListStore->get_iter(path);
+      if(iterRow)
       {
-        if(*iter == pColumn)
-          tree_col = col_index; //Found.
-
-        col_index++;
+        //Get the column:
+        int tree_col = 0;
+        int col_index = get_count_hidden_system_columns();
+  
+        typedef std::vector<Gtk::TreeView::Column*> type_vecTreeViewColumns;
+        type_vecTreeViewColumns vecColumns = m_TreeView.get_columns();
+        for(type_vecTreeViewColumns::const_iterator iter = vecColumns.begin(); iter != vecColumns.end(); iter++)
+        {
+          if(*iter == pColumn)
+            tree_col = col_index; //Found.
+  
+          col_index++;
+        }
+      
+        signal_user_activated().emit(iterRow, tree_col);
       }
-    
-      signal_user_activated().emit(iterRow, tree_col);
     }
   }
 
